#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    :  2021/11/5 13:26
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : qgisTools.py
# @Descr   : 对QGIS的操作
# @Software: PyCharm



"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: shpTools.py
@Date: Create in 2021/1/22 13:43
@Description: 对geojson文件的操作类
@ Software: PyCharm

===================================
"""

import json
import os

import geopandas as gpd
from osgeo import gdal, osr, ogr

from vgis_utils.vgis_file.fileTools import FileHelper


class GeoJsonHelper:

    def __init__(self):
        pass

    @staticmethod
    # 合并多个同类geojson
    def merge_geojsons(geojson_files_path, merge_result_file):
        index = 0
        all_data = {}
        encoding = FileHelper.get_file_encoding(geojson_files_path)
        for file_name in os.listdir(geojson_files_path):
            geojson_file = os.path.join(geojson_files_path, file_name)
            with open(geojson_file, 'r', encoding=encoding) as fp:
                each_data = json.load(fp)
            if index == 0:
                all_data = each_data
            else:
                all_data["features"] += each_data["features"]
        result_file = open(merge_result_file, "w")
        json.dump(all_data, result_file)

    # 将geojson转换为shp
    @staticmethod
    def convert_geojson_to_shp(geojson_path, shp_path):
        # 读取 GeoJSON 文件
        # 部分geosjon读取时报错：IllegalArgumentException: Points of LinearRing do not form a closed linestring--
        gdf = gpd.read_file(geojson_path, engine='pyogrio')
        # 保存为 Shapefile
        gdf.to_file(shp_path, driver='ESRI Shapefile', engine='pyogrio')

    @staticmethod
    def convert_geojson_to_shp2(geojson_path, shp_path):
        gdal.SetConfigOption("GDAL_FILENAME_IS_UTF8", "YES")
        gdal.SetConfigOption("SHAPE_ENCODING", "")
        geojson_ds = ogr.Open(geojson_path, 0)
        geojson_layer = geojson_ds.GetLayer(0)

        # 创建Shapefile输出数据源
        driver = ogr.GetDriverByName('ESRI Shapefile')
        # if driver.Exists(out_shp_file_path):
        #     driver.DeleteDataSource(out_shp_file_path)
        shp_ds = driver.CreateDataSource(shp_path)

        # 复制层并转换
        shp_layer = shp_ds.CopyLayer(geojson_layer, 'shp_layer')
        # 关闭数据源
        shp_ds.Destroy()
        geojson_ds.Destroy()

    # geojson里增加epsg
    @staticmethod
    def add_epsg_value_in_geojson(geojson_path, espg_value):
        encoding = FileHelper.get_file_encoding(geojson_path)
        data = json.load(open(geojson_path, encoding=encoding))
        if "crs" not in data:
            data['crs'] = {"type": "name", "properties": {"name": "urn:ogc:def:crs:EPSG::{}".format(espg_value)}}
        with open(geojson_path, 'w') as fp:
            json.dump(data, fp)

    # 获取geojson的几何类型和坐标信息
    @staticmethod
    def get_geojson_geometry_info(geojson_path):
        encoding = FileHelper.get_file_encoding(geojson_path)
        epsg_value, geometry_type = None, None
        f = open(geojson_path, encoding=encoding)
        data = json.load(f)
        if "crs" in data:
            epsg_info = data["crs"]["properties"]["name"]
            if "urn:ogc:def:crs:EPSG" in epsg_info:
                epsg_value = int(epsg_info.split("::")[1])
        if "features" in data:
            features = data["features"]
            if len(features) > 0:
                geometry_type = features[0]["geometry"]["type"]
                if epsg_value is None:
                    coordinates = str(features[0]["geometry"]["coordinates"])
                    print(coordinates)
                    x_coord = coordinates.split(",")[0].replace("[", "")
                    y_coord = coordinates.split(",")[1].replace("]", "")
                    x_coord = float(x_coord)
                    y_coord = float(y_coord)
                    # 这里只是简单做了判断
                    if x_coord <= 180 and y_coord <= 90:
                        epsg_value = 4326
                    else:
                        epsg_value = 3857

        return epsg_value, geometry_type

    @staticmethod
    def get_envlope_of_geojson(geojson_path):
        # 读取GeoJSON文件
        gdf = gpd.read_file(geojson_path, engine='pyogrio')
        # 获取坐标范围
        bounds = gdf.total_bounds
        return bounds


if __name__ == '__main__':
    geojson_path = "e:\\tttt-2.geoJson"
    # geojson_path = "e:\\110000.geoJson"
    shp_path = "e:\\tttt.shp"
    # GeoJsonHelper.convert_geojson_to_shp(geojson_path, shp_path)
    print(GeoJsonHelper.get_geojson_geometry_info(geojson_path))
    print(GeoJsonHelper.get_envlope_of_geojson(geojson_path)[0])



#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    :  2022/10/28 18:56
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : gisTools.py
# @Descr   : 
# @Software: PyCharm

# !/usr/bin/python3.9
# -*- coding: utf-8 -*-
"""
    地理中经常使用的数学计算，把地球简化成了一个标准球形，若是想要推广到任意星球能够改为类的写法，而后修改半径便可
"""
import math
import re

import numpy as np
from osgeo import gdal
from osgeo import osr
from pyproj import Proj, transform
from pyproj import Transformer
from shapely.geometry import Polygon


class GISHelper:
    earth_radius = 6370.856  # 地球平均半径，单位km，最简单的模型每每把地球当作完美的球形，这个值就是常说的RE
    math_2pi = math.pi * 2
    pis_per_degree = math_2pi / 360  # 角度一度所对应的弧度数，360对应2*pi

    def __init__(self):
        pass

    # 从投影信息获取UTM信息
    def get_utm_from_proj(self, proj_txt):
        # 适配不同的标识符
        # JB卫星: PROJCS[\"WGS_1984_UTM_Zone_54N\",GEOGCS
        # GJ卫星: PROJCRS[\"WGS 84 \ UTM zone 17N\",BASEGEOGCRS
        if proj_txt.strip().startswith("PROJCS"):
            proj = re.findall("PROJCS(.*?)GEOGCS", proj_txt)[0].lstrip("[")[:-1]
        elif proj_txt.strip().startswith("PROJCRS"):
            proj = re.findall("PROJCRS(.*?)BASEGEOGCRS", proj_txt)[0].lstrip("[")[:-1]
        return proj

    # 纬度差对应的KM
    def lat_degree2km(self, dif_degree, radius=earth_radius):
        """
        经过圆环求法，纯纬度上，度数差转距离(km)，与中间点所处的地球上的位置关系不大
        :param dif_degree: 度数差, 经验值0.0001对应11.1米的距离
        :param radius: 圆环求法的等效半径，纬度距离的等效圆环是经线环，因此默认都是earth_radius
        :return: 这个度数差dif_degree对应的距离，单位km
        """
        return radius * dif_degree * self.pis_per_degree

    # KM对应的纬度差
    def lat_km2degree(self, dis_km, radius=earth_radius):
        """
        经过圆环求法，纯纬度上，距离值转度数(diff)，与中间点所处的地球上的位置关系不大
        :param dis_km: 输入的距离，单位km，经验值111km相差约(接近)1度
        :param radius: 圆环求法的等效半径，纬度距离的等效圆环是经线环，因此默认都是earth_radius
        :return: 这个距离dis_km对应在纯纬度上差多少度
        """
        return dis_km / radius / self.pis_per_degree

    # 经度差对应的KM，需要提供中央纬度
    def lng_degree2km(self, dif_degree, center_lat):
        """
        经过圆环求法，纯经度上，度数差转距离(km)，纬度的高低会影响距离对应的经度角度差，具体表达式为：
        :param dif_degree: 度数差
        :param center_lat: 中心点的纬度，默认22为深圳附近的纬度值；为0时表示赤道，赤道的纬线环半径使得经度计算和上面的纬度计算基本一致
        :return: 这个度数差dif_degree对应的距离，单位km
        """
        # 修正后，中心点所在纬度的地表圆环半径
        real_radius = self.earth_radius * math.cos(center_lat * self.pis_per_degree)
        return self.lat_degree2km(dif_degree, real_radius)

    # KM对应的经度差，需要提供中央纬度
    def lng_km2degree(self, dis_km, center_lat):
        """
        纯经度上，距离值转角度差(diff)，单位度数。
        :param dis_km: 输入的距离，单位km
        :param center_lat: 中心点的纬度，默认22为深圳附近的纬度值；为0时表示赤道。
             赤道、中国深圳、中国北京、对应的修正系数分别约为： 1  0.927  0.766
        :return: 这个距离dis_km对应在纯经度上差多少度
        """
        # 修正后，中心点所在纬度的地表圆环半径
        real_radius = self.earth_radius * math.cos(center_lat * self.pis_per_degree)
        return self.lat_km2degree(dis_km, real_radius)

    # 两点（经纬度）之间的KM
    def ab_distance(self, a_lat, a_lng, b_lat, b_lng):
        """
        计算经纬度表示的ab两点的距离，这是种近似计算，当两点纬度差距不大时更为准确，产生近似的缘由也是来主要自于center_lat
        :param a_lat: a点纬度
        :param a_lng: a点经度
        :param b_lat: b点纬度
        :param b_lng: b点纬度
        :return:
        """
        center_lat = .5 * a_lat + .5 * b_lat
        lat_dis = self.lat_degree2km(abs(a_lat - b_lat))
        lng_dis = self.lng_degree2km(abs(a_lng - b_lng), center_lat)
        return math.sqrt(lat_dis ** 2 + lng_dis ** 2)

    # 构建矩形
    def get_rect_by_coords(self, coords):
        square = Polygon(
            ((coords[0], coords[2]), (coords[0], coords[3]), (coords[1], coords[3]), (coords[1], coords[2])))
        return square

    # 判断两个多边形是否相交
    # shaplely方法
    def is_overlap_of_polygons(self, poly1, poly2):
        return poly1.intersects(poly2) and not poly1.crosses(poly2) and not poly1.contains(poly2)

    # 通过经纬度得到像素坐标,根据地图高度宽度及地图经纬度范围
    def get_pixel_by_lnglat(self, point_x: float, point_y: float, map_boundary: str, map_witdh: int,
                            map_height: int) -> str:
        minx = float(map_boundary.split(",")[0])
        miny = float(map_boundary.split(",")[1])
        maxx = float(map_boundary.split(",")[2])
        maxy = float(map_boundary.split(",")[3])
        # 通过地图长宽和经纬度坐标获取像素坐标
        pixel_x = map_witdh / (maxx - minx) * (point_x - minx)
        pixel_y = map_height / (maxy - miny) * (maxy - point_y)
        return round(pixel_x), round(pixel_y)

    # 通过中心点坐标(经纬度)，图片宽度，图片高度，分辨率得到图片四个点坐标
    def get_box_by_center(self, data_centerpt: str, data_resolution: float, data_image_width: int,
                          data_image_height: int) -> str:

        data_minx = float(data_centerpt.split(",")[0]) - self.lng_km2degree(
            float(data_image_width) / 2 * float(data_resolution) / 1000, float(data_centerpt.split(",")[1]))
        data_maxx = float(data_centerpt.split(",")[0]) + self.lng_km2degree(
            float(data_image_width) / 2 * float(data_resolution) / 1000, float(data_centerpt.split(",")[1]))
        data_miny = float(data_centerpt.split(",")[1]) - self.lat_km2degree(
            float(data_image_height) / 2 * float(data_resolution) / 1000, 6370.856)
        data_maxy = float(data_centerpt.split(",")[1]) + self.lat_km2degree(
            float(data_image_height) / 2 * float(data_resolution) / 1000, 6370.856)
        return data_minx, data_maxx, data_miny, data_maxy

    # 转换坐标根据epsg
    @staticmethod
    def convert_points_from_epsg(point_list, from_epsg, to_epsg):
        from_prj = Proj(init='epsg:{}'.format(from_epsg))
        to_proj = Proj(init='epsg:{}'.format(to_epsg))
        new_point_list = []
        for point in point_list:
            x = point[0]
            y = point[1]
            x2, y2 = transform(from_prj, to_proj, x, y)
            new_point_list.append([x2, y2])
        return new_point_list

    # 4326坐标转3857坐标
    # point [[123.5,23.6]]
    @staticmethod
    def convert_point_from_wgs84_to_mercator(point):
        return_point = None
        # 地球半径
        earthRad = 6378137.0
        lng = point[0] * math.pi / 180 * earthRad
        param = point[1] * math.pi / 180
        lat = earthRad / 2 * math.log((1.0 + math.sin(param)) / (1.0 - math.sin(param)))
        return_point = [lng, lat]
        return return_point

    # 4326坐标转3857坐标
    # point [[350388,3957319.5]]
    @staticmethod
    def convert_point_from_mercator_to_wgs84(point):
        return_point = None
        lng = point[0] / 20037508.34 * 180
        lat = point[1] / 20037508.34 * 180
        lat = 180 / math.pi * (2 * math.atan(math.exp(lat * math.pi / 180)) - math.pi / 2)
        return_point = [lng, lat]
        return return_point

    # 投影坐标(米)到地理坐标（度）
    # proj_wkt： gdalinfo -json **.tif 得到json["coordinateSystem"]["wkt"]
    def convert_point_from_proj_to_geo(self, proj_wkt, x, y):
        # 地理坐标系是球面,投影坐标系是平面;
        # 地理坐标系的单位为“度”,投影坐标系的单位为“米”;
        prosrs = osr.SpatialReference()
        prosrs.ImportFromWkt(proj_wkt)
        geosrs = prosrs.CloneGeogCS()
        ct = osr.CoordinateTransformation(prosrs, geosrs)
        coords = ct.TransformPoint(x, y)
        # # 逆变换,实现投影到地理
        # p = Proj(projection)
        # lon, lat = p(x, y, inverse=True)
        return coords[:2][0], coords[:2][1]

    # 地理坐标（度）到投影坐标（米）
    # 输入的是要转的投影坐标系和经纬度
    def convert_point_from_geo_to_proj(self, proj_wkt, lon, lat):
        prosrs = osr.SpatialReference()
        prosrs.ImportFromWkt(proj_wkt)
        geosrs = prosrs.CloneGeogCS()
        ct = osr.CoordinateTransformation(geosrs, prosrs)
        coords = ct.TransformPoint(lon, lat)
        return coords[:2][0], coords[:2][1]

    # 投影坐标（米）到投影坐标（米）
    def convert_point_from_proj_to_proj(self, proj_wkt_source, proj_wkt_target, x, y):
        pass
        prosrs_source = osr.SpatialReference()
        prosrs_source.ImportFromWkt(proj_wkt_source)
        prosrs_target = osr.SpatialReference()
        prosrs_target.ImportFromWkt(proj_wkt_target)
        ct = osr.CoordinateTransformation(prosrs_source, prosrs_target)
        coords = ct.TransformPoint(x, y)
        return coords[:2][0], coords[:2][1]

    # 坐标转换（从一个坐标系到另一个坐标系），多个点
    # points:[[x1,y1],[x2,y2]....]
    # convert_method:proj_2_geo\geo_2_proj\proj_2_proj
    def convert_points_by_projwkt(self, points, proj_wkt, proj2_wkt, convert_method):
        try:
            point_list = []
            # 循环坐标点数组
            for point in points:
                x = point[0]
                y = point[1]
                if convert_method == "proj_2_geo":
                    x_convert, y_convert = self.convert_point_from_proj_to_geo(proj_wkt, x, y)
                if convert_method == "geo_2_proj":
                    x_convert, y_convert = self.convert_point_from_geo_to_proj(proj_wkt, x, y)
                if convert_method == "proj_2_proj":
                    x_convert, y_convert = self.convert_point_from_proj_to_proj(proj_wkt, proj2_wkt, x, y)

                point_list.append([x_convert, y_convert])

        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 输出异常信息，错误行号等
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

        return point_list

    # 针对WGS_1984_UTM_Zone_57N这种WGS84 UTM的分带投影做坐标转换
    # 从投影坐标米到地理坐标---单个点
    # point:[[350388,3957319.5]]
    # proj_wkt:"PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_northing\",0],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH],AUTHORITY[\"EPSG\",\"32654\"]]"
    def convert_point_from_utmproj_to_geo(self, point, proj_wkt, center_lon, center_lat):
        try:
            return_point = None
            point_x = point[0][0]
            point_y = point[0][1]
            proj_wkt = proj_wkt.upper()

            # 参数1：坐标系WKID WGS_1984_UTM_Zone_54N 对应 32654
            # 参数2：WGS84地理坐标系统 对应 4326
            # 如果是经纬度，就不用转
            if proj_wkt.strip().startswith("GEOGCS"):
                return_point = point
            else:
                proj = self.get_utm_from_proj(proj_wkt)
                epsg = self.get_utm_epsg_by_projection_info(proj, center_lon, center_lat)
                if epsg != -1:
                    print("转换前的espg:{}".format(epsg))
                    transformer = Transformer.from_crs("epsg:" + str(epsg), "epsg:4326")
                    lat, lon = transformer.transform(point_x, point_y)
                    return_point = [[lon, lat]]
                else:
                    print("坐标转换失败，通过投影没有获取到EPSG，请检查数据")
        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 打印输出异常信息
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

        return return_point

    # 转换坐标---从投影坐标米到地理坐标--多个点
    # points:[[350388,3957319.5],[352344,3966458.5]]
    # proj_wkt:"PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_northing\",0],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH],AUTHORITY[\"EPSG\",\"32654\"]]"
    def convert_points_from_utmproj_to_geo(self, points, proj_wkt, center_lon, center_lat):
        return_point_list = None
        try:
            res = ""
            # 参数1：坐标系WKID WGS_1984_UTM_Zone_54N 对应 32654
            # 参数2：WGS84地理坐标系统 对应 4326
            # 参数3: WGS84/UTM LJY,UTM无分带，强制对应为3857
            proj_wkt = proj_wkt.upper()
            print("投影信息：{}".format(proj_wkt))
            # 如果是经纬度，就不用转
            if proj_wkt.strip().startswith("GEOGCS"):
                return_point_list = points
            else:
                proj = self.get_utm_from_proj(proj_wkt)
                epsg = self.get_utm_epsg_by_projection_info(proj, center_lon, center_lat)
                print("转换前的espg:{}".format(epsg))
                transformer = Transformer.from_crs("epsg:" + str(epsg), "epsg:4326")
                point_list = []
                for point in points:
                    x = point[0]
                    y = point[1]
                    lat, lon = transformer.transform(x, y)
                    point_list.append([lon, lat])
                return_point_list = point_list
        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 输出异常信息，错误行号等
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数
        return return_point_list

    # 针对WGS_1984_UTM_Zone_57N这种WGS84 UTM的分带投影做坐标转换
    # 地理坐标（经纬度）到投影坐标（米）--单个点
    def convert_point_from_geo_to_utmproj(self, point, proj_wkt, center_lon, center_lat):
        return_point = None
        try:
            lon = point[0][0]
            lat = point[0][1]
            proj_wkt = proj_wkt.upper()
            # 参数1：坐标系WKID WGS_1984_UTM_Zone_54N 对应 32654
            # 参数2：WGS84地理坐标系统 对应 4326
            # 投影1：PROJCS[\"WGS_1984_UTM_Zone_54N\",GEOGCS
            # 投影2：PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS
            proj = self.get_utm_from_proj(proj_wkt)
            epsg = self.get_utm_epsg_by_projection_info(proj, center_lon, center_lat)
            if epsg != -1:
                print("转换前的espg:{}".format(epsg))
                # 建立转换器
                transformer = Transformer.from_crs("epsg:4326", "epsg:" + str(epsg))
                # lat, lon = transformer.transform(point_x, point_y)
                point_x, point_y = transformer.transform(lat, lon)
                #             # 初始化4326和3857的pyproj投影对象
                #             p4326 = Proj(init='epsg:4326')  # WGS 84
                #             p3857 = Proj(init='epsg:3857')  # Web 墨卡托
                #             # 使用transform函数进行坐标转换
                #             tif_minx, tif_maxy= transform(p4326, p3857, tif_minx, tif_maxy)
                return_point = [[point_x, point_y]]
            else:
                print("坐标转换失败，通过投影没有获取到EPSG，请检查数据")
        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 打印输出异常信息
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

        return return_point

    # 转换坐标---地理坐标（经纬度）到投影坐标（米）---多个点
    # point:[[350388,3957319.5],[352344,3966458.5]]
    # proj_txt:"PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_northing\",0],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH],AUTHORITY[\"EPSG\",\"32654\"]]"
    def convert_points_from_geo_to_utmproj(self, points, proj_wkt, center_lon, center_lat):
        return_point_list = None
        try:
            res = ""
            # 参数1：坐标系WKID WGS_1984_UTM_Zone_54N 对应 32654
            # 参数2：WGS84地理坐标系统 对应 4326
            proj_wkt = proj_wkt.upper()
            # 通过正则表达式得到坐标系
            # 投影1：PROJCS[\"WGS_1984_UTM_Zone_54N\",GEOGCS
            # 投影2：PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS
            proj = self.get_utm_from_proj(proj_wkt)
            epsg = self.get_utm_epsg_by_projection_info(proj, center_lon, center_lat)
            print("转换前的espg:{}".format(epsg))
            transformer = Transformer.from_crs("epsg:4326", "epsg:" + str(epsg))
            point_list = []
            # 循环坐标点数组
            for point in points:
                lon = point[0]
                lat = point[1]
                x, y = transformer.transform(lat, lon)
                point_list.append([x, y])
            return_point_list = point_list
        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 输出异常信息，错误行号等
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数
        return return_point_list

    # 通过投影信息和中央经纬度得到EPSG
    def get_utm_epsg_by_projection_info(self, projection, center_lon, center_lat):
        epsg_code = -1
        try:
            print("转换前的proj:{}".format(projection))
            utm_code = projection.replace(" ", "").replace("_", "").lower()
            # wgs84 utm分带投影的epsg计算
            if "utmzone" in utm_code and ("wgs84" in utm_code or "wgs1984" in utm_code):
                if len(re.findall("zone(.*?)n", utm_code)) > 0:
                    # 北半球 epsg=32600+wgs84 utm分度带号
                    utm_code_zone = int(re.findall("zone(.*?)n", utm_code)[0])
                    epsg_code = 32600 + utm_code_zone
                if len(re.findall("zone(.*?)s", utm_code)) > 0:
                    # 南半球 epsg=32700+wgs84 utm分度带号
                    utm_code_zone = int(re.findall("zone(.*?)s", utm_code)[0])
                    epsg_code = 32700 + utm_code_zone
            # utm没有分带投影的epsg计算
            elif "utm" in utm_code and "utmzone" not in utm_code:
                # 东半球
                if float(center_lon) > 0:
                    utm_code_zone = 31 + int(float(center_lon) / 6)
                # 西半球
                if float(center_lon) < 0:
                    utm_code_zone = 30 - int(abs(float(center_lon) / 6))
                # 北半球
                if float(center_lat) > 0:
                    epsg_code = 32600 + utm_code_zone
                # 南半球
                if float(center_lat) < 0:
                    epsg_code = 32700 + utm_code_zone

        except Exception as exp:
            print("通过投影获取EPSG失败：" + str(exp))
            # epsg_code=3857
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数
        return int(epsg_code)

    # 计算多边形坐标的四至范围
    # 多边形坐标为[[point1x,point1y],[point2x,point2y],...[pointnx,pointny],[point1x,point1y]]
    @staticmethod
    def get_box_of_polygon_points(points):
        points_np = np.array(points)

        cols_min = np.min(points_np, axis=0)
        cols_max = np.max(points_np, axis=0)
        minx = cols_min[:1].tolist()[0]
        miny = cols_min[1:2].tolist()[0]
        maxx = cols_max[:1].tolist()[0]
        maxy = cols_max[1:2].tolist()[0]
        # minx = np.min(points_np[:, 0])
        # maxx = np.max(points_np[:, 0])
        # miny = np.min(points_np[:, 1])
        # maxy = np.max(points_np[:, 1])
        return minx, miny, maxx, maxy

    # 得到tif文件的投影信息
    def get_source_projection(self, tif_file):
        dataset = gdal.Open(tif_file)
        return dataset.GetProjection()

    @staticmethod
    def get_circle_polygons_by_center_and_radius( center, radius,num_points):
        """
        根据中心点和半径，生成圆形的多边形
        :param center: 中心点坐标，格式为[x,y]
        :param radius: 半径
        :return: 圆形的多边形
        """
        # 计算圆形的顶点坐标
        points = []
        for i in range(num_points + 1):
            angle = i * (360 / num_points)
            x = center[0] + (radius * math.cos(math.radians(angle)))
            y = center[1] + (radius * math.sin(math.radians(angle)))
            points.append([x, y])
        return points




if __name__ == '__main__':
    gisHelper = GISHelper()
    # print(gisHelper.lng_km2degree(0.5, 45))
    # dataset = gdal.Open("E:\\data\\123.tif")
    # print(dataset.GetProjection())
    # print(dataset.GetGeoTransform())

    print("---------------------------------")

    # dataset = gdal.Open("E:\\data\\sar1234\\1234.tiff")
    # print(dataset.GetProjection())
    # print(dataset.GetGeoTransform())

    # 得到原始tif的投影信息
    # source_projection = gisHelper.get_source_projection("E:\\data\\123.tif")
    # source_projection = gisHelper.get_source_projection("E:\\desktop\\DEM\\ribenxibu.tif")
    # # 输入原始tif的投影信息，需要转换的点的投影坐标（米）
    # result = gisHelper.convert_points_from_meter_to_lnglat(source_projection, 350388, 3957319.5)
    # # 输出 转换后的经纬度值（度）
    # print("lon:" + str(result[0]))
    # print("lat:" + str(result[1]))
    point = [113, 23]
    print(gisHelper.convert_point_from_wgs84_to_mercator(point))
    point = [12604705.942522367, 2632018.637586424]
    print(gisHelper.convert_point_from_mercator_to_wgs84(point))



"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: poiTools.py
@Date: Create in 2021/1/28 9:59
@Description: POI数据查询
http://lbsyun.baidu.com/index.php?title=webapi/guide/webservice-placeapi
@ Software: PyCharm
===================================
"""

import json
import math
from time import sleep

import requests

from vgis_utils.vgis_list.listTools import ListHelper


class POIOperater:

    def __init__(self, baidu_key_list, baidu_city_code_txt):
        self.baidu_key_list = baidu_key_list
        self.baidu_city_code_txt = baidu_city_code_txt
        self.baidu_key_index = 0
        self.baidu_key = self.baidu_key_list[self.baidu_key_index]
        self.baidu_key_status = "配额正常"
        self.headers = {
            "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.81 Safari/537.36",
        }

    # 连接百度接口，判断状态,如果许可配额用完，切换到另外一个配合
    def make_baidu_connect_and_return_data(self, query_word, region_name, page_num):
        sleep(1)
        url = 'http://api.map.baidu.com/place/v2/search?query=%s&region=%s&output=json&ak=%s&page_size=20&page_num=%s&coord_type=wgs84ll' % (
            query_word, region_name, self.baidu_key, page_num)
        res = requests.get(url, headers=self.headers)
        res.encoding = 'utf-8-sig'
        data = json.loads(res.text)
        if data.get("status") == 0:
            self.baidu_key_status = "配额正常"
            return data
        # 超出配额限制
        elif data.get("status") == 301 or data.get("status") == 302:
            print("------------------------------{}配额用完".format(self.baidu_key))
            self.baidu_key_index += 1
            if self.baidu_key_index <= len(self.baidu_key_list) - 1:
                self.baidu_key = self.baidu_key_list[self.baidu_key_index]
                self.baidu_key_status = "配额用完"
                print("-------------------------------切换到新许可{}".format(self.baidu_key))
                url = 'http://api.map.baidu.com/place/v2/search?query=%s&region=%s&output=json&ak=%s&page_size=20&page_num=%s&coord_type=wgs84ll' % (
                    query_word, region_name, self.baidu_key, page_num)
                res = requests.get(url, headers=self.headers)
                res.encoding = 'utf-8-sig'
                data = json.loads(res.text)
                if data.get("status") == 0:
                    self.baidu_key_status = "切换的配额正常"
                    return data
                elif data.get("status") == 301 or data.get("status") == 302:
                    # 这个时候，会漏掉本次查询的
                    self.baidu_key_status = "切换的配额用完"
                    return None
            else:
                print("-------------------------------所有的配额都用完，请第二天再使用")
                return None

    # 触发调用百度接口，获取结果对象
    def toogle_query_baidu_poi(self, query_word, region_name, page_num):
        return self.make_baidu_connect_and_return_data(query_word, region_name, page_num)

    # 触发调用百度接口，获取结果类型
    def get_query_baidu_data_type(self, query_word, region_name, page_num):
        data = self.make_baidu_connect_and_return_data(query_word, region_name, page_num)
        return data['result_type']

    # 得到city_type下面的name,有可能是省，也可能是地市
    def get_result_name_list_of_city_type(self, query_word, region_name, page_num):
        result_name_list = []
        data = self.make_baidu_connect_and_return_data(query_word, region_name, page_num)
        for t in range(0, len(data['results'])):
            result_region_name = data['results'][t].get('name')
            result_name_list.append(result_region_name)
        return result_name_list

    # 获取百度查询需要的地市名称，根据百度接口返回结果调整
    # 不支持任意值的区域名称
    def get_area_region_of_baidu_need(self, query_word, region_name):
        need_region_list = []
        if region_name == "全国":
            need_region_list = self.get_all_city_of_country()
        else:
            if self.get_query_baidu_data_type(query_word, region_name, 0) == "poi_type":
                # 针对只有一页的poi_type
                if self.get_query_baidu_data_type(query_word, region_name, 1) is None:
                    need_region_list.append(region_name)
                else:
                    # 针对两页多的poi_type
                    if self.get_query_baidu_data_type(query_word, region_name, 1) == "poi_type":
                        need_region_list.append(region_name)
                    # 第一页是poi_type,第二页是city_type, 如区域为四川，关键词是煤炭厂
                    elif self.get_query_baidu_data_type(query_word, region_name, 1) == "city_type":
                        result_name_list = self.get_result_name_list_of_city_type(query_word, region_name, 1)
                        for result_name in result_name_list:
                            need_region_list.append(result_name)
            elif self.get_query_baidu_data_type(query_word, region_name, 0) == "city_type":
                # 煤炭厂 嘉峪关市
                if self.get_query_baidu_data_type(query_word, region_name, 1) == "poi_type":
                    pass
                else:
                    result_name_list2 = self.get_result_name_list_of_city_type(query_word, region_name, 1)
                    for result_name2 in result_name_list2:
                        need_region_list.append(result_name2)
        return need_region_list

    # 从百度地图官网上下载的百度地图city_code文件中获取全国的城市列表
    # 后期百度地图的city_code更新后，同步更新这个txt文件接口
    # 下载地址：http://lbsyun.baidu.com/index.php?title=open/%E5%BC%80%E5%8F%91%E8%B5%84%E6%BA%90
    def get_all_city_of_country(self):
        city_list = []
        with open(self.baidu_city_code_txt, 'r', encoding='UTF-8') as file:
            city_code_file = file.read()
            current_row = 0
            line_list = city_code_file.split("\n")
            for line in line_list:
                current_row += 1
                if current_row == 1:
                    continue
                else:
                    city_list.append(line.split(",")[1])
        return city_list

    # 单次调用百度地图接口获取结果数据
    def get_poi_data_from_baidu(self, query_word, region_name):
        print("对{}内的{}进行采集".format(region_name, query_word))
        query_result = []
        data = self.toogle_query_baidu_poi(query_word, region_name, 0)
        # 即使是正常地市名，也会查询出city_code，如煤炭厂，嘉峪关市，第一页是city_code,第二页是poi_type，但没有数值
        # 上述情况有可能是嘉峪关市没有煤炭厂的原因
        if data is not None and data['result_type'] == "poi_type":
            total = data['total']
            pageSize = math.ceil(total / 20)
            if (pageSize):
                for i in range(0, pageSize):
                    print("采集第{}/{}页数据".format(str(i + 1), str(pageSize)))
                    data = self.toogle_query_baidu_poi(query_word, region_name, i)
                    if data is not None:
                        for t in range(0, len(data['results'])):
                            obj = {}
                            try:
                                obj['名称'] = data['results'][t].get('name')
                            except:
                                obj['名称'] = ""
                                pass
                            try:
                                address = data['results'][t].get('province') + ',' + data['results'][t].get(
                                    'city') + ',' + \
                                          data['results'][t].get('area') + ',' + data['results'][t].get('address')
                                obj['地址'] = address
                            except:
                                obj['地址'] = ""
                                pass
                            try:
                                obj['省份'] = data['results'][t].get('province')
                            except:
                                obj['省份'] = ""
                                pass
                            try:
                                obj['地市'] = data['results'][t].get('city')
                            except:
                                obj['地市'] = ""
                                pass
                            try:
                                obj['区县'] = data['results'][t].get('area')
                            except:
                                obj['区县'] = ""
                                pass
                            try:
                                obj['电话'] = data['results'][t].get('telephone')
                            except:
                                obj['电话'] = ""
                                pass
                            try:
                                obj['经度'] = data['results'][t].get('location').get('lng')
                            except:
                                obj['经度'] = ""
                                pass
                            try:
                                obj['纬度'] = data['results'][t].get('location').get('lat')
                            except:
                                obj['纬度'] = ""
                                pass
                            query_result.append(obj)
        return query_result

    # 批量调用百度POI接口获取所有结果
    # TODO:获取高德，四维的POI
    def get_all_poi_data_batch_by_baidu(self, query_word, query_region):
        all_result_list = []
        need_region_list = self.get_area_region_of_baidu_need(query_word, query_region)
        for need_region in need_region_list:
            try:
                if self.get_poi_data_from_baidu(query_word, need_region) is not None and len(
                        self.get_poi_data_from_baidu(query_word, need_region)) > 0:
                    all_result_list.append(self.get_poi_data_from_baidu(query_word, need_region))
            except:
                pass
            continue
        all_result_list_converse = ListHelper.convert_all_list_into_one_list(all_result_list)
        return all_result_list_converse



"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: geoserverTools.py
@Date: Create in 2021/1/28 13:14
@Description: GeoServer操作类
@ Software: PyCharm
===================================
"""
import base64
import os

import requests
import requests as req
from vgis_gis.shpTools import ShpFileOperator
# from vgis_rs.tifTools import TifFileOperator
from vgis_utils.vgis_datetime.datetimeTools import DateTimeHelper
from requests.auth import HTTPBasicAuth


class GeoserverOperatoer:

    # 初始化
    def __init__(self, geoserver_http_address, username, password):
        # "http://192.168.3.77:8085/geoserver"
        self.geoserver_http_address = geoserver_http_address
        self.username = username
        self.password = password
        self.serect = username + ":" + password
        self.basic_auth = "Basic " + str(base64.b64encode(self.serect.encode("utf-8")), "utf-8")

    def get_workspace(self):
        url = '{}/rest/workspaces'.format(self.geoserver_http_address)
        response = requests.get(url, auth=HTTPBasicAuth(self.username, self.password))
        return response.text

    # 创建工作空间
    def create_workspace(self, workspace_name):
        url = '{}/rest/workspaces'.format(self.geoserver_http_address)
        payload = {'name': workspace_name}
        response = requests.post(url, json=payload, auth=HTTPBasicAuth(self.username, self.password))
        return response.status_code == 201

    # 创建数据存储
    # 栅格存储
    # /workspaces/{workspaceName}/coveragestores/{coveragestoreName}/coverages
    # 矢量存储
    # /workspaces/{workspaceName}/datastores/{datastoreName}/featuretypes,
    # WMSStores 用于从其他 Web Map Service (WMS) 服务器获取数据并将其集成到 GeoServer 中
    # /workspaces/{workspaceName}/wmsstores/{wmsstoreName}/wmslayers
    # WMTSStores 用于从其他 Web Map Tile Service (WMTS) 服务器获取瓦片数据并将其集成到 GeoServer
    # /workspaces/{workspaceName}/wmtsstores/{wmststoreName}/wmtslayers
    def create_datastore(self, workspace_name, datastore_name, file_path):
        url = '{}/rest/workspaces/{workspace_name}/datastores'.format(self.geoserver_http_address)
        headers = {'Content-Type': 'application/zip'}
        files = {'file': open(file_path, 'rb')}
        response = requests.post(url, auth=HTTPBasicAuth(self.username, self.password), headers=headers, files=files)
        return response.status_code == 201

    #


    # 删除栅格存储
    def delete_coveragestore(self, workspace_name, store_name):
        # 删除栅格存储
        # curl -v -u admin:geoserver -X DELETE http://127.0.0.1:8080/geoserver/rest/workspaces/jimu/coveragestores/uuuu?recurse=true
        cmd = "curl -v -u {} -X DELETE {}/rest/workspaces/{}/coveragestores/{}?recurse=true".format(self.serect,
                                                                                                    self.geoserver_http_address,
                                                                                                    workspace_name,
                                                                                                    store_name)
        print(cmd)
        os.system(cmd)

    # 删除栅格存储
    def delete_datastore(self, workspace_name, store_name):
        # 删除矢量存储
        # curl -v -u admin:geoserver -X DELETE http://127.0.0.1:8080/geoserver/rest/workspaces/jimu/datastores/tttt?recurse=true
        cmd = "curl -v -u {} -X DELETE {}/rest/workspaces/{}/datastores/{}?recurse=true".format(self.serect,
                                                                                                    self.geoserver_http_address,
                                                                                                    workspace_name,
                                                                                                    store_name)
        print(cmd)
        os.system(cmd)

    # 清除服务缓存
    def clear_geoserver_cache(self):
        cmd = 'curl -v -u {} -X POST {}/rest/reset -H  "accept: application/json" -H  "content-type: application/json"'.format(
            self.serect, self.geoserver_http_address)
        print(cmd)
        os.system(cmd)

    # 发布geoserver tif 图层服务-WMS
    # tif名称要和layer_name对应上
    # 将tip文件上传到geoserver的data目录进行发布
    def publish_raster_layer_service(self, tif_path, workspacename, layer_name):
        print("发布栅格图层服务{}".format(layer_name))
        url = self.geoserver_http_address + "/rest/workspaces/" + workspacename + "/coveragestores/" + layer_name + "/file.geotiff"
        # Authorization信息在postman测试用例里看
        headers = {
            'Content-type': 'image/tiff',
            'Authorization': self.basic_auth
        }
        file = open(tif_path, 'rb')
        payload = file.read()
        response = req.request("PUT", url, headers=headers, data=payload)
        print(response.text)

    # tif名称要和layer_name对应上
    # 按照tif原有路径进行发布
    def publish_tif_layer_service_v2(self, tif_path, layer_name, workspacename):
        print("发布栅格图层服务{}".format(layer_name))
        # windowsw文件路径
        # file://C:\data\test\jiujiang2018_landuse_newcode.tif
        # linux文件路径
        # file:///data/shapefiles/rivers/rivers.tif
        cmd = 'curl -v -u {}:{} -XPUT -H "Content-type: text/plain"   -d "file://{}"   {}/rest/workspaces/{}/coveragestores/{}/external.geotiff'.format(
            self.username, self.password, tif_path, self.geoserver_http_address, workspacename, layer_name)
        print(cmd)
        os.system(cmd)

    # 去除栅格图层的黑边
    def clear_black_raster_layer_service(self, workspacename, datastore_name, layer_name):

        print("去除黑边")
        url = "{}/rest/workspaces/{}/coveragestores/{}/coverages/{}".format(self.geoserver_http_address, workspacename,
                                                                            datastore_name, layer_name)

        payload = "<coverage>   \r\n    <parameters>\r\n    <entry>\r\n      <string>InputTransparentColor</string>\r\n      <string>#000000</string>\r\n    </entry>\r\n  </parameters>\r\n</coverage>\r\n"
        headers = {
            'Content-Type': 'application/xml',
            'Authorization': self.basic_auth
        }

        response = req.request("PUT", url, headers=headers, data=payload)

        print(response.text)

    # 判断是否有图层存在
    def is_has_layer(self, layer_name):
        ishas = False
        url = "{}/rest/layers/{}".format(self.geoserver_http_address, layer_name)
        payload = ""
        headers = {
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        if "No such layer" not in response.text:
            ishas = True

        return ishas

    # 发布geoserver shp 图层服务--WMS
    # TODO:有问题，针对部分shp不能自动识别空间范围和srs
    # 将shp文件上传到geoserver的data目录进行发布
    def publish_shp_layer_service(self, shp_path, layer_name, workspacename, layer_style_color):
        print("发布矢量图层服务{}".format(layer_name))
        url = self.geoserver_http_address + "/rest/workspaces/" + workspacename + "/datastores/" + layer_name + "/file.shp"
        file = open(shp_path, 'rb')
        payload = file.read()
        headers = {
            'Content-type': 'application/zip',
            'Authorization': self.basic_auth
        }
        response = req.request("PUT", url, headers=headers, data=payload)
        print(response.text)

    # shp名称要和layer_name对应上
    # 按照shp原有路径进行发布
    def publish_shp_layer_service_v2(self, shp_path, layer_name, workspacename):
        print("发布矢量图层服务{}".format(layer_name))
        # windowsw文件路径
        # file://C:\data\test\jiujiang2018_landuse_newcode.shp
        # linux文件路径
        # file:///data/shapefiles/rivers/rivers.shp
        cmd = 'curl -v -u {}:{} -XPUT -H "Content-type: text/plain"   -d "file://{}"   {}/rest/workspaces/{}/datastores/{}/external.shp'.format(
            self.username, self.password, shp_path, self.geoserver_http_address, workspacename, layer_name)
        print(cmd)
        os.system(cmd)

    # 设置矢量图层的空间范围和坐标
    # 针对发布图层没有自动识别的情况
    def set_layer_coord_and_srs(self, layer_name, workspacename, minx, maxx, miny, maxy, epsg):
        cmd = ' curl -v -u {}:{} -XPUT -H "Content-type:text/xml" -d "<layer><bounds><minx>{}</minx><maxx>{}</maxx><miny>{}</miny><maxy>{}</maxy><crs>ESPG:{}</crs></bounds></layer>" {}/rest/layers/{}:{}'.format(
            self.username, self.password,
            minx, maxx, miny, maxy, epsg, self.geoserver_http_address, workspacename, layer_name)
        print(cmd)
        os.system(cmd)

    # 发布postgis矢量图层
    def publish_postgis_feature_service(self, layer_name, workspace_name, pg_datastore_name):
        cmd = 'curl -v -u {} -XPOST -H "Content-type: text/xml" -d  "<featureType><name>{}</name></featureType>" {}/rest/workspaces/{}/datastores/{}/featuretypes '.format(
            self.serect, layer_name, self.geoserver_http_address, workspace_name, pg_datastore_name)
        print(cmd)
        os.system(cmd)

    # 发布geoserver图层组服务
    def publish_layer_group_service(self, layer_group_name, workspacename, layer_name_list):
        print("发布图层组服务{}".format(layer_group_name))
        base_layer_name = layer_name_list[0]
        base_layer_info = self.get_layer_info(base_layer_name)
        layergroup_url = self.geoserver_http_address + "/rest/layergroups"
        layergroup_payload = "<layerGroup><name>" + layer_group_name + "</name><mode>SINGLE</mode><title>" + layer_group_name + "</title><workspace><name>" + workspacename + "</name></workspace>"
        layergroup_payload += "<layers>"
        for i in range(len(layer_name_list)):
            if layer_name_list[i] != None and layer_name_list[i].strip() != "":
                layergroup_payload += "<layer>" + layer_name_list[i] + "</layer>"
        layergroup_payload += "</layers>"
        layergroup_payload += "<styles>"
        for w in range(len(layer_name_list)):
            if layer_name_list[i] != None and layer_name_list[i].strip() != "":
                layergroup_payload += "<style>" + self.get_layer_style(layer_name_list[w]) + "</style>"
        layergroup_payload += "</styles>"
        layergroup_payload += "<bounds><minx>" + str(base_layer_info.get("minx")) + "</minx><maxx>" + str(
            base_layer_info.get("maxx")) + "</maxx><miny>" + str(base_layer_info.get("miny")) + "</miny><maxy>" + str(
            base_layer_info.get("maxy")) + "</maxy><crs>" + base_layer_info.get("srs") + "</crs></bounds></layerGroup>"
        print(layergroup_payload)
        layergroup_headers = {
            'Content-type': 'text/xml',
            'Authorization': self.basic_auth
        }
        curl_post_layergroup_response = req.request("POST", layergroup_url, headers=layergroup_headers,
                                                    data=layergroup_payload)
        print(curl_post_layergroup_response.text)

    def get_layer_full_info(self, layer_name):
        # curl -X GET http://localhost:12686/geoserver/rest/layers/jimu:ningbo_airplane_label_google -H  "accept: application/xml" -H  "content-type: application/xml"
        url = self.geoserver_http_address + "/rest/layers/" + layer_name
        payload = {}
        headers = {
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        print(response.text)

    # 得到图层信息，包括空间范围、坐标系等
    def get_layer_info(self, layer_name):
        url = self.geoserver_http_address + "/rest/layers/" + layer_name
        payload = {}
        headers = {
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        # print(response.text)
        response.encoding = 'utf-8'
        results = response.json()
        layer_prop_href = results.get("layer").get("resource").get("href")
        style_name = results.get("layer").get("defaultStyle").get("name")
        layer_response = req.request("GET", layer_prop_href, headers=headers)
        layer_response.encoding = 'utf-8'
        layer_results = layer_response.json()
        srs = layer_results.get("coverage").get("srs")
        minx = layer_results.get("coverage").get("nativeBoundingBox").get("minx")
        miny = layer_results.get("coverage").get("nativeBoundingBox").get("miny")
        maxx = layer_results.get("coverage").get("nativeBoundingBox").get("maxx")
        maxy = layer_results.get("coverage").get("nativeBoundingBox").get("maxy")
        obj = {}
        obj["style_name"] = style_name
        obj["srs"] = srs
        obj["minx"] = minx
        obj["miny"] = miny
        obj["maxx"] = maxx
        obj["maxy"] = maxy
        return obj

    # 获取图层的样式名称
    def get_layer_style(self, layer_name):
        url = self.geoserver_http_address + "/rest/layers/" + layer_name
        payload = {}
        headers = {
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        # print(response.text)
        response.encoding = 'utf-8'
        results = response.json()
        style_name = results.get("layer").get("defaultStyle").get("name")
        return style_name

    # 删除geoserver图层服务
    def delete_layer_service(self, layer_name):
        print("删除图层服务{}".format(layer_name))
        curl_delete_layer_url = self.geoserver_http_address + "/rest/layers/" + layer_name
        curl_delete_layer_payload = {}
        curl_delete_layerg_headers = {
            'Authorization': self.basic_auth
        }
        curl_delete_layer_response = req.request("DELETE", curl_delete_layer_url,
                                                 headers=curl_delete_layerg_headers,
                                                 data=curl_delete_layer_payload)
        print(curl_delete_layer_response.text)

    # 删除GeoServer图层
    def delete_layer_service2(self, layer_name):
        cmd = "curl -v -u {}:{} -XDELETE {}/rest/layers/{}?recurse=true".format(self.username, self.password,
                                                                                self.geoserver_http_address,
                                                                                layer_name)
        os.system(cmd)

    # 删除geoserver图层组服务
    def delete_layer_group_service(self, layer_group_name, workspacename):
        print("删除图层组服务{}".format(layer_group_name))
        if layer_group_name != None and layer_group_name.strip() != "":
            curl_delete_layergroup_url = self.geoserver_http_address + "/rest/layergroups/" + workspacename + ":" + layer_group_name
            curl_delete_layergroup_payload = {}
            curl_delete_layergroup_headers = {
                'Authorization': self.basic_auth
            }
            curl_delete_layergroup_response = req.request("DELETE", curl_delete_layergroup_url,
                                                          headers=curl_delete_layergroup_headers,
                                                          data=curl_delete_layergroup_payload)
            print(curl_delete_layergroup_response.text)

    # 动态生成geoserver样式
    def make_layer_style(self, style_color, workspacename):
        style_name = "temp_style_" + DateTimeHelper.get_uuid()
        print("创建新样式{}".format(style_name))
        url = self.geoserver_http_address + "/rest/workspaces/" + workspacename + "/styles?name=" + style_name
        payload = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><sld:StyledLayerDescriptor xmlns=\"http://www.opengis.net/sld\" xmlns:sld=\"http://www.opengis.net/sld\" xmlns:gml=\"http://www.opengis.net/gml\" xmlns:ogc=\"http://www.opengis.net/ogc\" version=\"1.0.0\">\r\n  <sld:NamedLayer>\r\n    <sld:Name>polygon fill</sld:Name>\r\n    <sld:UserStyle>\r\n      <sld:Name>polygon fill</sld:Name>\r\n      <sld:Title>polygon fill</sld:Title>\r\n      <sld:Abstract>polygon fill</sld:Abstract>\r\n      <sld:FeatureTypeStyle>\r\n        <sld:Name>polygon fill</sld:Name>\r\n        <sld:Rule>\r\n          <sld:PolygonSymbolizer>\r\n            <sld:Fill>\r\n              <sld:CssParameter name=\"fill\">#66FF66</sld:CssParameter>\r\n              <sld:CssParameter name=\"fill-opacity\">0</sld:CssParameter>\r\n            </sld:Fill>\r\n            <sld:Stroke>\r\n              <sld:CssParameter name=\"stroke\">" + style_color + "</sld:CssParameter>\r\n            </sld:Stroke>\r\n          </sld:PolygonSymbolizer>\r\n        </sld:Rule>\r\n      </sld:FeatureTypeStyle>\r\n    </sld:UserStyle>\r\n  </sld:NamedLayer>\r\n</sld:StyledLayerDescriptor>\r\n"
        headers = {
            'accept': 'application/json',
            'content-type': 'application/vnd.ogc.sld+xml',
            'Authorization': self.basic_auth
        }
        response = req.request("POST", url, headers=headers, data=payload)
        print(response.text)
        return style_name

    # 删除geoerver样式
    def delete_layer_style(self, style_name, workspacename):
        print("删除样式{}".format(style_name))
        if style_name != None and style_name.strip() != "":
            url = self.geoserver_http_address + "/rest/styles/" + workspacename + ":" + style_name
            payload = {}
            headers = {
                'Authorization': self.basic_auth
            }
            response = req.request("DELETE", url, headers=headers, data=payload)
            print(response.text)

    # 给图层设置样式
    def set_style_to_layer(self, layer_name, style_name):
        print("为图层{}设置样式{}".format(layer_name, style_name))
        url = self.geoserver_http_address + "/rest/layers/" + layer_name
        payload = "<layer>\r\n    <defaultStyle>\r\n        <name>" + style_name + "</name>\r\n    </defaultStyle>\r\n    <enabled>true</enabled>\r\n</layer>"
        headers = {
            'Content-type': 'text/xml',
            'Authorization': self.basic_auth
        }
        response = req.request("PUT", url, headers=headers, data=payload)
        print(response.text)

    # 给图层设置样式
    def set_style_to_layer2(self, workspace_name, layer_name, style_name):
        cmd = 'curl -v -u {} -X PUT {}/rest/workspaces/{}/layers/{} -H "accept: application/json" -H "content-type: application/xml"  -d "<layer><defaultStyle><name>{}</name></defaultStyle></layer>"'.format(
            self.serect, workspace_name, layer_name, style_name)
        print(cmd)
        os.system(cmd)

    # 获取gwc切片方案信息
    def get_grid_set_info(self, grid_set_name):
        url = "{}/gwc/rest/gridsets/{}".format(self.geoserver_http_address, grid_set_name)
        payload = {}
        headers = {
            'accept': 'application/json',
            'content-type': 'application/json',
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        print(response.text)
        return response.text

    # 设置MVT为输出格式
    def set_mvt_format(self, workspace_name, layer_name):

        print("为图层{}设置MVTT输出{}".format(layer_name))
        # url = '{}/rest/workspaces/{}/layers/{}'.format(self.geoserver_http_address, workspace_name, layer_name)
        url = self.geoserver_http_address + "/rest/layers/" + workspace_name + ":" + layer_name
        payload = "<layer><tileCaching><cacheType>ALL</cacheType><gridSubdivision>0</gridSubdivision><formats><format>image/png</format><format>application/vnd.mapbox-vector-tile</format></formats></tileCaching></layer>"
        headers = {
            'Content-type': 'text/xml',
            'Authorization': self.basic_auth
        }
        response = req.request("PUT", url, headers=headers, data=payload)
        print(response.text)
        return response.status_code == 201

    # 开始切片
    def run_seed_tiles(self, workspacename, layername, bbox, srs):
        import requests

        # GeoWebCache REST API的URL
        gwc_url = '{}/gwc/rest/layer'.format(self.geoserver_http_address)

        # 需要切片的图层名称，格式为'工作区:图层名'
        layer_name = '{}:{}'.format(workspacename, layername)

        # 创建会话
        session = requests.Session()
        session.auth = (self.username, self.password)

        # 开始切片的请求
        seed_request = {
            "seed": {
                "recurse": True,
                # "bbox": "-180,-90,180,90",  # 切片的边界框，根据需要修改
                # "srs": "EPSG:4326"  # 坐标系统，根据需要修改
                "bbox": bbox,
                "srs": srs
            }
        }

        # 发送切片请求
        response = session.post(f'{gwc_url}/{layer_name}/seed', json=seed_request)

        # 检查响应
        if response.status_code == 200:
            print("切片请求已提交。")
        else:
            print("切片请求失败。", response.text)

        # 关闭会话
        session.close()


if __name__ == '__main__':
    geoserverOperatoer = GeoserverOperatoer('http://localhost:8080/geoserver', 'admin', 'geoserver')
    workspace = "jimu"

    # 发布栅格图层
    # tif_path = "c:/data/TW2015_4326.tif"
    # layer = os.path.split(tif_path)[1].split(".")[0]
    # geoserverOperatoer.publish_raster_layer_service(tif_path, workspace, layer)
    # geoserverOperatoer.clear_black_raster_layer_service(workspace, layer, layer)
    # epsg=TifFileOperator.get_epsg_of_tif(tif_path)
    # raster_tiles_url = "http://{}/gwc/service/wmts?layer={}:{}&style=&tilematrixset=EPSG:{}&Service=WMTS&Request=GetTile&Version=1.0.0&Format=image:vnd.jpeg-png&TileMatrix=EPSG:{}:{z}&TileCol={x}&TileRow={y}".format(geoserverOperatoer.geoserver_http_address,workspace, layer, epsg, epsg)

    # 发布矢量图层
    shp_path = "c:/data/test/ningbo_airplane_label_google.shp"
    layer = os.path.split(shp_path)[1].split(".")[0]
    left, right, down, up = ShpFileOperator.get_layer_envlope(shp_path)
    import os

    os.environ[
        'PROJ_LIB'] = "C:\\Users\\63241\\miniconda3\\envs\\django422_py310\\Lib\\site-packages\\pyproj\\proj_dir\\share\\proj"
    from vgis_gis.shpTools import ShpFileOperator

    epsg = ShpFileOperator.get_epsg_of_shp_v2(shp_path)
    geoserverOperatoer.publish_shp_layer_service_v2(shp_path, layer, workspace)
    geoserverOperatoer.set_layer_coord_and_srs(layer, workspace, left, right, down, up, epsg)

    geojson_vector_tiles_url = "http://{}/gwc/service/wmts?REQUEST=GetTile&SERVICE=WMTS&VERSION=1.0.0&LAYER={}:{}&STYLE=&TILEMATRIX=EPSG:{}:{z}&TILEMATRIXSET=EPSG:{}&FORMAT=application/json;type=geojson&TILECOL={x}&TILEROW={y}".format(
        geoserverOperatoer.geoserver_http_address, workspace, layer, epsg, epsg)
    mvt_vector_tiles_url = "http://{}/gwc/service/wmts?REQUEST=GetTile&SERVICE=WMTS&VERSION=1.0.0&LAYER={}:{}&STYLE=&TILEMATRIX=EPSG:{}:{z}&TILEMATRIXSET=EPSG:{}&FORMAT=application/vnd.mapbox-vector-tile&TILECOL={x}&TILEROW={y}".format(
        geoserverOperatoer.geoserver_http_address, workspace, layer, epsg, epsg)

# WMTS服务查看页面
# http://localhost:12686/geoserver/gwc/demo/jimu:tw2015?gridSet=EPSG:4326&format=image/vnd.jpeg-png
# http://localhost:8080/geoserver/gwc/demo/jimu:ningbo_airplane_label_google?gridSet=EPSG:4326&format=application/vnd.mapbox-vector-tile

# WMTS瓦片URL
# jpeg-png 栅格瓦片
# http://{ip}:{port}/geoserver/gwc/service/wmts?layer={workspacename}:{layername}&style=&tilematrixset=EPSG:{epsg}&Service=WMTS&Request=GetTile&Version=1.0.0&Format=image:vnd.jpeg-png&TileMatrix=EPSG:{epsg}:{z}&TileCol={x}&TileRow={y}
# geojson 矢量瓦片
# http://{ip}:{port}/geoserver/gwc/service/wmts?REQUEST=GetTile&SERVICE=WMTS&VERSION=1.0.0&LAYER={workspacename}:{layername}&STYLE=&TILEMATRIX=EPSG:{epsg}:{z}&TILEMATRIXSET=EPSG:{epsg}&FORMAT=application/json;type=geojson&TILECOL={x}&TILEROW={y}
# topojson 矢量瓦片
# http://{ip}:{port}/geoserver/gwc/service/wmts?REQUEST=GetTile&SERVICE=WMTS&VERSION=1.0.0&LAYER={workspacename}:{layername}&STYLE=&TILEMATRIX=EPSG:{epsg}:{z}&TILEMATRIXSET=EPSG:{epsg}&FORMAT=application/json;type=topojson&TILECOL={x}&TILEROW={y}
# utfgrid 矢量瓦片
# http://{ip}:{port}/geoserver/gwc/service/wmts?REQUEST=GetTile&SERVICE=WMTS&VERSION=1.0.0&LAYER={workspacename}:{layername}&STYLE=&TILEMATRIX=EPSG:{epsg}:{z}&TILEMATRIXSET=EPSG:{epsg}&FORMAT=application/json;type=utfgrid&TILECOL={x}&TILEROW={y}
# mvt矢量瓦片
# http://{ip}:{port}/geoserver/gwc/service/wmts?REQUEST=GetTile&SERVICE=WMTS&VERSION=1.0.0&LAYER={workspacename}:{layername}&STYLE=&TILEMATRIX=EPSG:{epsg}:{z}&TILEMATRIXSET=EPSG:{epsg}&FORMAT=application/vnd.mapbox-vector-tile&TILECOL={x}&TILEROW={y}

# wms瓦片
#  mvt瓦片 矢量瓦片
# http://{ip}:{port}/geoserver/{worksapcename}/wms?service=WMTS&request=GetTile&version=1.0.0&layer={layername}&style=&tilematrixset=EPSG:{epsg}&format=application/vnd.mapbox-vector-tile&tilematrix=EPSG:{epsg}:{z}&tilerow={y}&tilecol={x}
